/******************************************************************************
  $Id: PTG_Func2dVec.cc,v 1.2 2004/09/03 11:43:57 schatz Exp $
******************************************************************************/


#include <math.h>

#include "PTG_Func2dVec.h"
#include "PTG_Func2dScalar.h"
#include "globals.h"

void PTG_Func2dVec::GradMid(const PTG_Func2dScalar& s)
{
  const double D = 1.0/(2.0*Delta());
  const int iNx = SizeX() - 1;
  const int iNy = SizeY() - 1;
  // const int prevy0 = (duneglobals::periodic_y())?(iNy-1):iNy;
//   const int prevx0 = (duneglobals::periodic_x())?(iNx-1):iNx;
//   const int nextyiNy = (duneglobals::periodic_y())?1:0;
//   const int nextxiNx = (duneglobals::periodic_x())?1:0;
  
  const int prevy0 = iNy; const int prevx0 = iNx;
  const int nextyiNy = 0; const int nextxiNx = 0;

  PTG_Func2dVec TempVec;
  TempVec.Create( SizeX(), SizeY(), Delta() );

  // ---- inside ----
  for (int y=1; y < iNy; y++) {
    for (int x=1; x < iNx; x++) {
      TempVec(x,y)[0] = opDMid( s(x-1,y), s(x+1,y), D );
      TempVec(x,y)[1] = opDMid( s(x,y-1), s(x,y+1), D );
    }
  }

  // ---- y boundary ----
  for (int x=1; x < iNx; x++) {
    TempVec(x,0)[0]   = opDMid( s(x-1,0), s(x+1,0), D );
    TempVec(x,0)[1]   = opDMid( s(x,prevy0), s( x ,1), D );

    TempVec(x,iNy)[0] = opDMid( s(x-1,iNy), s(x+1,iNy), D );
    TempVec(x,iNy)[1] = opDMid( s(x,iNy-1), s(x,nextyiNy), D );
  }

  // ---- x boundary ----
  for (int y=1; y < iNy; y++) {
    TempVec(0,y)[0]   = opDMid( s(prevx0,y), s(1,y), D );
    TempVec(0,y)[1]   = opDMid( s(0,y-1), s(0,y+1), D );

    TempVec(iNx,y)[0] = opDMid( s(iNx-1,y), s(nextxiNx,y), D );
    TempVec(iNx,y)[1] = opDMid( s(iNx,y-1), s(iNx,y+1), D );
  }

  // ---- edges ----
  TempVec(0,0)[0]     = opDMid( s(prevx0,0), s(1,0), D );
  TempVec(0,0)[1]     = opDMid( s(0,prevy0), s(0,1), D );

  TempVec(0,iNy)[0]   = opDMid( s(prevx0,iNy), s(1,iNy), D );
  TempVec(0,iNy)[1]   = opDMid( s(0,iNy-1), s(0,nextyiNy), D );

  TempVec(iNx,0)[0]   = opDMid( s(iNx-1,0), s(nextxiNx,0), D );
  TempVec(iNx,0)[1]   = opDMid( s(iNx,prevy0), s(iNx,1), D );

  TempVec(iNx,iNy)[0] = opDMid( s(iNx-1,iNy), s(nextxiNx,iNy), D );
  TempVec(iNx,iNy)[1] = opDMid( s(iNx,iNy-1), s(iNx,nextyiNy), D );

  // output (*this)(x,y)
  *this = TempVec;

}


void PTG_Func2dVec::GradUpWind(const PTG_Func2dScalar& s, const PTG_Func2dVec& u)
{
  const double D = 1.0/Delta();
  const int iNx = SizeX() - 1;
  const int iNy = SizeY() - 1;
  // const int prevy0 = (duneglobals::periodic_y())?(iNy-1):iNy;
//   const int prevx0 = (duneglobals::periodic_x())?(iNx-1):iNx;
//   const int nextyiNy = (duneglobals::periodic_y())?1:0;
//   const int nextxiNx = (duneglobals::periodic_x())?1:0;
  const int prevy0 = iNy; const int prevx0 = iNx;
  const int nextyiNy = 0; const int nextxiNx = 0;

  PTG_Func2dVec TempVec;
  TempVec.Create( SizeX(), SizeY(), Delta() );


  // ---- inside ----
  for (int y=1; y < iNy; y++) {
    for (int x=1; x < iNx; x++) {
      TempVec(x,y)[0] = opDUpWind( s(x-1,y), s(x,y), s(x+1,y), D, u(x,y)[0] );
      TempVec(x,y)[1] = opDUpWind( s(x,y-1), s(x,y), s(x,y+1), D, u(x,y)[1] );
    }
  }

  // ---- y boundary ----
  for (int x=1; x < iNx; x++) {
    TempVec(x,0)[0] = opDUpWind( s(x-1,0), s(x,0), s(x+1,0), D, u(x,0)[0] );
    TempVec(x,0)[1] = opDUpWind( s(x,prevy0), s(x,0), s( x ,1), D, u(x,0)[1] );

    TempVec(x,iNy)[0] = opDUpWind( s(x-1,iNy), s(x,iNy), s(x+1,iNy), D, u(x,iNy)[0] );
    TempVec(x,iNy)[1] = opDUpWind( s(x,iNy-1), s(x,iNy), s(x,nextyiNy), D, u(x,iNy)[1] );
  }

  // ---- x boundary ----
  for (int y=1; y < iNy; y++) {
    TempVec(0,y)[0] = opDUpWind( s(prevx0,y), s(0,y), s(1,y), D, u(0,y)[0] );
    TempVec(0,y)[1] = opDUpWind( s(0,y-1), s(0,y), s(0,y+1), D, u(0,y)[1] );

    TempVec(iNx,y)[0] = opDUpWind( s(iNx-1,y), s(iNx,y), s(nextxiNx,y), D, u(iNx,y)[0] );
    TempVec(iNx,y)[1] = opDUpWind( s(iNx,y-1), s(iNx,y), s(iNx,y+1), D, u(iNx,y)[1] );
  }

  // ---- edges ----
  TempVec(0,0)[0] = opDUpWind( s(prevx0,0), s(0,0), s(1,0), D, u(0,0)[0] );
  TempVec(0,0)[1] = opDUpWind( s(0,prevy0), s(0,0), s(0,1), D, u(0,0)[1] );

  TempVec(0,iNy)[0] = opDUpWind( s(prevx0,iNy), s(0,iNy), s(1,iNy), D, u(0,iNy)[0] );
  TempVec(0,iNy)[1] = opDUpWind( s(0,iNy-1), s(0,iNy), s(0,nextyiNy), D, u(0,iNy)[1] );

  TempVec(iNx,0)[0] = opDUpWind( s(iNx-1,0), s(iNx,0), s(nextxiNx,0), D, u(iNx,0)[0] );
  TempVec(iNx,0)[1] = opDUpWind( s(iNx,prevy0), s(iNx,0), s(iNx,1), D, u(iNx,0)[1] );

  TempVec(iNx,iNy)[0] = opDUpWind( s(iNx-1,iNy), s(iNx,iNy), s(nextxiNx,iNy), D, u(iNx,iNy)[0] );
  TempVec(iNx,iNy)[1] = opDUpWind( s(iNx,iNy-1), s(iNx,iNy), s(iNx,nextyiNy), D, u(iNx,iNy)[1] );

  // output (*this)(x,y)
  *this = TempVec;

}


double PTG_Func2dVec::GetMaxAbs()
{
  const int iNx = SizeX();
  const int iNy = SizeY();

  double dMax2 = 0.;
  for (int y=0; y < iNy; y++) {
    for (int x=0; x < iNx; x++) {
      double dX = (*this)(x,y)[0];
      double dY = (*this)(x,y)[1];
      double dAbs2 = dX*dX + dY*dY;
      if (dMax2 < dAbs2) {
	dMax2 = dAbs2;
      }
    }
  }
  return sqrt(dMax2);
}


/*!  Copies an array of possibly different size into this one, using linear
  interpolation.  The corner values are matched exactly, and the rectangle in
  between is stretched or shrunk as necessary.  */

void PTG_Func2dVec::copyscale( const PTG_Func2dVec source )
{
  double xscale, yscale, xweight, yweight;
  int x, y, basex, basey;

  xscale= (double)(source.SizeX()-1)/(double)(SizeX()-1);
  yscale= (double)(source.SizeY()-1)/(double)(SizeY()-1);
  for( x=0; x < SizeX(); ++x )
    for( y=0; y < SizeY(); ++y ) {
      xweight= (double)x * xscale;
      basex= (int)trunc(xweight);
      xweight -= trunc(xweight);
      yweight= (double)y * yscale;
      basey= (int)trunc(yweight);
      yweight -= trunc(yweight);
      // We have to be careful here, because trying to use the general formula
      // for the last x or y would access an illegal location.  We ignore small
      // deviations from integer indices to be on the safe side.
      if( xweight < 1e-20 )
        if( yweight < 1e-20 )
	  (*this)(x, y)= source(basex, basey);
	else
	  (*this)(x, y)= source(basex, basey) + xweight * 
		    (source(basex+1, basey) - source(basex, basey));
      else
        if( yweight < 1e-20 )
	  (*this)(x, y)= source(basex, basey) + yweight * 
		    (source(basex, basey+1) - source(basex, basey));
	else
	  (*this)(x, y)= source(basex, basey) + 
	  xweight * (source(basex+1, basey) - source(basex, basey)) +
	  yweight * (source(basex, basey+1) - source(basex, basey) +
            xweight * (source(basex+1, basey+1) - source(basex, basey+1)
	              - source(basex+1, basey) + source(basex, basey)) );
    }
}



/*!  Computes the average and standard deviation of the difference of x and y
  components between neighbouring grid sites.  These values can be used to
  determine where this vecor field has discontinuities.  Zero differences are
  ignored so the standard deviation is not influenced by a constant plane
  surrounding the structure to be investigated.  */

void PTG_Func2dVec::get_diff_stat( double *avg_x, double *avg_y, 
				    double *sigma_x, double *sigma_y )
{
  double diff, du_x_sum, du_x_sqrsum, du_y_sum, du_y_sqrsum;
  int x, y, n_du_x, n_du_y;

  du_x_sum= du_x_sqrsum= 0.0;
  du_y_sum= du_y_sqrsum= 0.0;
  n_du_x= n_du_y= 0;
  for( x= 0; x< SizeX()-1; ++x ) {
    for( y= 0; y< SizeY()-1; ++y ) {
      diff= fabs((*this)(x, y)[0] - (*this)(x+1, y)[0]);
      if( diff ) {
        ++n_du_x;
	du_x_sum += diff;
	du_x_sqrsum += diff*diff;
      }
      diff= fabs((*this)(x, y)[0] - (*this)(x, y+1)[0]);
      if( diff ) {
        ++n_du_x;
	du_x_sum += diff;
	du_x_sqrsum += diff*diff;
      }
      diff= fabs((*this)(x, y)[1] - (*this)(x+1, y)[1]);
      if( diff ) {
        ++n_du_y;
	du_y_sum += diff;
	du_y_sqrsum += diff*diff;
      }
      diff= fabs((*this)(x, y)[1] - (*this)(x, y+1)[1]);
      if( diff ) {
        ++n_du_y;
	du_y_sum += diff;
	du_y_sqrsum += diff*diff;
      }
    }
    diff= fabs((*this)(x, y)[0] - (*this)(x+1, y)[0]);
    if( diff ) {
      ++n_du_x;
      du_x_sum += diff;
      du_x_sqrsum += diff*diff;
    }
    diff= fabs((*this)(x, y)[1] - (*this)(x+1, y)[1]);
    if( diff ) {
      ++n_du_y;
      du_y_sum += diff;
      du_y_sqrsum += diff*diff;
    }
  }
  for( y= 0; y< SizeY()-1; ++y ) {
    diff= fabs((*this)(x, y)[0] - (*this)(x, y+1)[0]);
    if( diff ) {
      ++n_du_x;
      du_x_sum += diff;
      du_x_sqrsum += diff*diff;
    }
    diff= fabs((*this)(x, y)[1] - (*this)(x, y+1)[1]);
    if( diff ) {
      ++n_du_y;
      du_y_sum += diff;
      du_y_sqrsum += diff*diff;
    }
  }
  if( n_du_x ) {
    du_x_sum /= n_du_x;
    du_x_sqrsum /= n_du_x;
    *avg_x= du_x_sum;
    *sigma_x= sqrt(du_x_sqrsum - du_x_sum*du_x_sum);
  }
  else {
    *avg_x= 0.0;
    *sigma_x= 0.0;
  }
  if( n_du_y ) {
    du_y_sum /= n_du_y;
    du_y_sqrsum /= n_du_y;
    *avg_y= du_y_sum;
    *sigma_y= sqrt(du_y_sqrsum - du_y_sum*du_y_sum);
  }
  else {
    *avg_y= 0.0;
    *sigma_y= 0.0;
  }
}



