/******************************************************************************
    $Id: flux.h,v 1.7 2005/01/26 08:25:03 schatz Exp $
******************************************************************************/

#ifndef FLUX_H
#define FLUX_H

#include "func.h"
#include "globals.h"

/*!  Base class of classes computing sand flux for 3D simulations.  */

class flux3d : public dunedata
{
public:
  flux3d(const dunepar &p) : dunedata(p) { 
    m_rho.Create(duneglobals::nx(), duneglobals::ny(), duneglobals::dx()); }
  virtual ~flux3d() {}

  virtual void calc( TFktVec& flux, const TFktScal& h, const TFktScal& h_nonerod, const TFktVec& tau )= 0;

  static flux3d *create( const dunepar &p );

protected:
  TFktScal m_rho;
};


/*!  Base class of flux computing classes which also return the
  erosion/deposition rate.  */

class flux3dgamma : public flux3d
{
public:
  flux3dgamma(const dunepar &p) : flux3d(p) {}
  virtual ~flux3dgamma() {}

  virtual void calc( TFktVec& flux, const TFktScal& h, const TFktScal& h_nonerod, const TFktVec& tau, TFktScal& gamma )= 0;
  virtual void calc( TFktVec& flux, const TFktScal& h, const TFktScal& h_nonerod, const TFktVec& tau )= 0;

  static flux3dgamma *create( const dunepar &p );
};


/*!  Dummy flux calculation class for 3D simulations.  Always sets flux to 0.
  For saving time when testing something else.  */

class flux3d_dummy : public flux3dgamma
{
public:
  flux3d_dummy(const dunepar &p) : flux3dgamma(p) {}
  virtual ~flux3d_dummy() {}

  virtual void calc( TFktVec& flux, const TFktScal& h, const TFktScal& h_nonerod, const TFktVec& tau, TFktScal& gamma );
  virtual void calc( TFktVec& flux, const TFktScal& h, const TFktScal& h_nonerod, const TFktVec& tau );
  virtual void save_arrays() {}
};


#endif // FLUX_H
