#include <stdlib.h>

#include "vec.h"
#include "initsurfalea.h"

////////////////////////////////////////
// Plain initial surface with fluctuations
//

void CInitSurfAlea::Create(PTG_Func2dScalar& f)
{
  const int iNX = f.SizeX();
  const int iNY = f.SizeY();
  const double dDelta = f.Delta();

  double dH, dFluc;
  dH = m_height;

  dFluc = m_fluctuations;

  if (dFluc > dH) {
    dFluc = dH;
  }

  for (int iX=0; iX<iNX; iX++) {
    for (int iY=0; iY<iNY; iY++) {
      f[iX][iY] = dH;
    }
  }

  int iNGauss;
  if (dFluc <= 0.0) {
    dH = 0.0;
    dFluc = 0.0;
    iNGauss = 0;
  } else {
    iNGauss = iNX*iNY / ( 80 * (int) (ceil(dFluc*dFluc)));
  }

  const double dRandMax = RAND_MAX;
  cout << iNGauss << " small gaussian hills of max. height " << dFluc << endl << endl;

  for (int i=0; i<iNGauss; i++) {
    const double dSigma = 25.0 + (random() * ( 10.0 / dRandMax));
    const double dHi = dFluc * ((double) (2.0 * random()) / dRandMax - 1.0);
    const double dC = sqrt(2*M_PI) * dSigma * dHi;
    const int x0 = iNX * (int) (random() * dDelta / dRandMax);
    const int y0 = iNY * (int) (random() * dDelta / dRandMax);
    const vec2 vx0( x0, y0 );
    cout << dSigma << " " << dHi << " " << x0 << " " << y0 << endl;

    for (int iX=0; iX<iNX; iX++) {
      for (int iY=0; iY<iNY; iY++) {
	const vec2 x(iX * dDelta, iY * dDelta);
	f[iX][iY] += dC / (dSigma * 2.0 / M_2_SQRTPI) * 
		    exp( -  ((x[0]-vx0[0])*(x[0]-vx0[0]) + 
			(x[1]-vx0[1])*(x[1]-vx0[1])) / (2*dSigma*dSigma) );
	if (f[iX][iY] < 0.0) {
	  f[iX][iY] = 0.0;
	}
      }
    }
  }
}


