/******************************************************************************
  $Id: initsurfgauss.cc,v 1.8 2004/10/01 13:37:28 schatz Exp $
******************************************************************************/

#include "initsurfgauss.h"
#include "globals.h"

////////////////////////////////////////
// class CInitSurfGauss
//

CInitSurfGauss::CInitSurfGauss(const dunepar& par, string prefix)
{
  m_h= par.getrequired<double>(prefix+"gauss.h_0", 0.0);
  m_H= par.getrequired<double>(prefix+"gauss.plain_height", 0.0);
  m_xslope= par.getrequired<double>(prefix+"gauss.plain_xslope", 0.0);
  m_yslope= par.getrequired<double>(prefix+"gauss.plain_yslope", 0.0);
  m_volume= par.getrequired<double>(prefix+"gauss.volume", 0.0);
  m_sigma= par.getdefault<double>(prefix+"gauss.sigma", 2.0*m_h);
  m_Xsigma= par.getdefault<double>(prefix+"gauss.Xsigma", 0);
  m_x0= par.getdefault<double>(prefix+"gauss.x_0", 
	      (double)(duneglobals::nx()-1) * duneglobals::dx() / 2.);
  m_y0= par.getdefault<double>(prefix+"gauss.y_0", 
		   (double)(duneglobals::ny()-1) * duneglobals::dx() / 2.);

  m_xscale = par.getdefault(prefix+"gauss.scale_x", 1.0);
  m_yscale = par.getdefault(prefix+"gauss.scale_y", 1.0);
  m_maxslope = tan(par.getdefault<double>(prefix+"gauss.max.slope", 30.0)*M_PI/180.);
}


/*!  Puts a number of Gaussian hills into the height profile \a array.  */

void CInitSurfGauss::init_2d_scal(TFktScal& array)
{
  for (int iX=0; iX< array.SizeX(); iX++)
    for (int iY=0; iY< array.SizeY(); iY++)
      array(iX, iY) = 0.0;

    if(m_volume!=0){
        const double B= m_maxslope*exp(0.5);
	m_sigma= pow( m_volume / (2*M_PI*B), 1./3.);
	m_h= B * m_sigma;
    }
    else if(m_sigma==0){
       m_sigma= m_h*exp(-0.5)/m_maxslope;
    }
    for (int iX=0; iX< array.SizeX(); iX++)
      for (int iY=0; iY< array.SizeY(); iY++) {
        const double X= (iX * array.Delta() - m_x0);
	const double dX= (X>0 ? 1. : -1.)*m_Xsigma*array.SizeX()*array.Delta();
	const double Y= (iY * array.Delta() - m_y0);
	double Ratio2= ((X*X/(m_xscale*m_xscale)+Y*Y/(m_yscale*m_yscale)<dX*dX)? 0 : ((X-dX)*(X-dX)/(m_xscale*m_xscale) + (Y-dX)*(Y-dX)/(m_yscale*m_yscale))/(2 *
	m_sigma*m_sigma));

	const double plane= m_H + (m_xslope * iX + m_yslope *
	iY) * array.Delta();

	array(iX, iY)= m_h * exp( - Ratio2 ) + (plane >= 0 ? plane : 0);
      }
   /*double N= m_h;
   const double R= 3.;
   double x0=m_xslope, y0=m_yslope;
   int rep=0;
   int *Pos=new int[18];
   Pos[1]=0; Pos[2]=0;
   Pos[3]=-1; Pos[4]=-1;
   Pos[5]=-1; Pos[6]=1;
   Pos[7]=-2; Pos[8]=0;
   Pos[9]=-2; Pos[10]=-2;
   Pos[11]=-2; Pos[12]=2;
   Pos[13]=-3; Pos[14]=-1;
   Pos[15]=-3; Pos[16]=1;
   Pos[17]=-4; Pos[18]=0;
*/
   /*
   Pos[1]=0; Pos[2]=1;
   Pos[3]=0; Pos[4]=-1;
   Pos[5]=-1; Pos[6]=1;
   Pos[7]=-1; Pos[8]=-1;
   Pos[9]=-2; Pos[10]=1;
   Pos[11]=-2; Pos[12]=-1;
   Pos[13]=-3; Pos[14]=1;
   Pos[15]=-3; Pos[16]=-1;


   Pos[1]=0; Pos[2]=2;
   Pos[3]=0; Pos[4]=1;
   Pos[5]=0; Pos[6]=-1;
   Pos[7]=0; Pos[8]=-2;
   Pos[9]=-1; Pos[10]=1;
   Pos[11]=-1; Pos[12]=0;
   Pos[13]=-1; Pos[14]=-1;
   Pos[15]=-1; Pos[16]=-2;
*/
/*
   for(int n=1; n< N+1; n++){
   for (int iX=0; iX< array.SizeX(); iX++)
      for (int iY=0; iY< array.SizeY(); iY++) {
        double x= iX*array.Delta()-m_x0-x0*R*Pos[2*n-1];
	double y= iY*array.Delta()-m_y0-y0*R*Pos[2*n];
 	if(y > -R && y < R){
	 if(x<= 0 && x > -2.*sqrt(R*R-y*y)){
	 	double z2= R*R-y*y-0.25*x*x;
		if(z2 > 0 && z2 < (0.86*x)*(0.86*x)) array(iX, iY)= sqrt(z2)+m_H;
	 }
	}
      }
  }*/
}

