/******************************************************************************
  $Id: pde.h,v 1.10 2005/04/08 11:48:45 schatz Exp $
******************************************************************************/

#if !defined(__PDE_H__)
#define __PDE_H__

#include <vector>

#include "globals.h"

//////////////////////////////////////////////////
// partial differential equation (PDE) class
//

#include "vec.h"
#include "func.h"


// class prototypes
class shear;

//class flux;
class flux3d_stationary;
class influx;

class avalanche;

class CAnalyze;

class CSaveFields;

class dunepar;

class CBoundary;

/*!  This is the main loop of Sauermann's original version of the program.  It
  creates all the objets needed for the simulation (in the constructor) and
  calls their calculation routines in turn in Calc().  */

class CPde
{
public:
  // construction
  CPde( dunepar& P );
  ~CPde();

  void Create(dunepar& Para);

  // attributes
  double GetGridSize() const { return m_dDX; }

  // ---- init ----

  void SetInitConf( dunepar& Para );

  // ---- calc ----

  // all
  void Calc(int t);


  // ---- analyze ----
  void SaveFields(int t, bool bDebug=false);


public:
  // ---- functions ----

  TFktScal m_h;

  /*!  Height profile of the non-erodable surface under the sand.  */
  TFktScal m_h_nonerod;
  
  TFktScal m_dh;
  
  /*!  Vegetation profile */
  TFktScal m_veget;
  TFktScal m_veget_pos;
  TFktScal m_h_aux;

  TFktVec m_flux;
  TFktScal m_ex;
  
  TFktVec m_tau;

  //CBoundary* m_pactBoundary;

private:

  dunepar& m_P;
  
  shear*       m_pactShear;
  
  //flux*	     m_pactFlux;
  flux3d_stationary*	m_pactFlux;
  influx*	m_influx;
  
  avalanche*         m_pactIterAval;

  CAnalyze*          m_pactAnalyze;

  CSaveFields*       m_pactSave;
  
  CBoundary* m_pactBoundary;

  bool m_bPeriodic;
  bool m_calc_flux_in_eq_out;
  bool m_calc_dh;
  bool m_calc_veget;
  bool m_analyze;
  bool m_bMsg;
  bool m_vol_correct;
  bool m_calc_shift_back;
  bool m_shift_back_cm0;
  bool m_shift_back_cmT;
  bool m_save_h_only;

  int m_iNX;
  int m_iNY;

  double m_dDX; // grid distance
  double m_dDtMax;
  double m_dDhMax;

  double m_veget_limit, m_h_veget, m_rhofactor, m_veget_conc, m_t, m_veget_erosion, m_veget_deposition, m_veget_grow_eros;
  bool m_veget_root;
  // ---- parameter ----

  double m_dRhoSand;

  // ---- boundary ----

  double m_q_s_x0;

  // ----

  double m_dVol0;
  double m_dCenterOfMass0;
  double m_dCenterOfMassOld;
  int m_iShiftX;
  double m_dH0;
  
  int veget_start;

  double m_dInFlux;
  double m_dOutFlux;
  double m_dOutFlux_x;
  double m_dOutFlux_y;
};







#endif

