#ifndef __SHEAR_H__
#define __SHEAR_H__


#include "vec.h"
#include "func.h"
#include "globals.h"

class CSaveFields;
class sepbubble;


////////////////////////////////////////////////////////////
// shear
//
class shear : public dunedata
{
public:
  shear(const dunepar& P);
  virtual ~shear();

  virtual void Calc(const TFktScal& h, TFktVec& tau, int t);

  virtual bool Save() { return false; }
  virtual void SaveFunc(CSaveFields& actSave);

  virtual void save_arrays();

  void set_ustar( double u_star );

protected:
  virtual void CalcPertTau(const TFktScal& h, TFktVec& tau) = 0;
  /*!  The shear velocity on a flat plain corresponding to the current wind
    speed.  */
  double m_u_star;
  /*!  The unperturbed shear stress corresponding to m_u_star.  */
  double m_dTau0;

private:

  TFktScal m_stall;
  TFktScal m_hSepBub;
  sepbubble* m_pSepBub;

  TFktVec m_TauP;

  bool m_bTauYZero;
  bool m_bSepbub;
  /*!  This seems to give the decrease in shear stress after the brink of the
    slip face.  That would determine the length of the region after the brink
    where the sand blown over the brink is deposited.  This variable is given
    by the parameter sepbub.tau.  */
  double m_tau_sepbub;
};

////////////////////////////////////////////////////////////
// shear_veget
//

class shear_veget : public dunedata
{
public:
  shear_veget(const dunepar& P);
  virtual ~shear_veget();

  virtual void Calc(const TFktScal& h, const TFktVec& veget, TFktVec& tau, int t); 

  virtual bool Save() { return false; }
  virtual void SaveFunc(CSaveFields& actSave);
  
  virtual void save_arrays();
  
  static shear_veget *create( const dunepar &p );
  void set_ustar( double u_star );
  
protected:
  virtual void CalcPertTau(const TFktScal& h, const TFktVec& veget, TFktVec& tau) = 0;
  /*!  The shear velocity on a flat plain corresponding to the current wind
    speed.  */
  double m_u_star;
  /*!  The unperturbed shear stress corresponding to m_u_star.  */
  double m_dTau0;

private:
  double m_sep_slope;

  TFktScal m_stall;
  TFktScal m_hSepBub;
  sepbubble* m_pSepBub;

  TFktVec m_TauP;
  
  bool m_bTauYZero;
  bool m_bSepbub;
  /*!  This seems to give the decrease in shear stress after the brink of the
    slip face.  That would determine the length of the region after the brink
    where the sand blown over the brink is deposited.  This variable is given
    by the parameter sepbub.tau.  */
  double m_tau_sepbub;
  
  bool m_calc_veget, m_u_ast_t_veget;
  double m_k, m_h_veget, m_vegetfactor_1, m_vegetfactor_2;
};

#endif
